// exemple de programme utilisant la bibliotheque vkey
// ***************************************************
// ecrit par Cordier yves le 18/03/2017
//

#include <vkey.h>

// ce programme permet d'acquerir un nombre decimal
// avec nbavantvirgule maxi de chrifres et nbdecimale correspondant au nombre de chiffres après la virgule
// 

#define brocheclavier A0
#define vitessevisu 9600
#define ledcontrol 13;
#define nbavantvirgule 5
#define nbdecimale 2

#define valide 12
#define pointdecimal 11
#define touchezero 10
#define toucheun 1
#define touchedeux 2
#define touchetrois 3
#define touchequatre 4
#define touchecinq 5
#define touchesix 6
#define touchesept 7
#define touchehuit 8
#define toucheneuf 9
#define toucherien 0

Vkey key = Vkey(brocheclavier); // création de l'objet key connecté sur broche A0

// *************************************************************
// prend un nombre entier au clavier 
// contenant au maxi nbdigitmax symboles
// si prisezero est vrai, on compte les premiers zero 
// si non, on ne prend en compte que les chiffres significatifs

long prendEntier(int nbdigitsmax, boolean prisezero)
{
	// les poids numerique associés aux touches du clavier
	static int table[10] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 0}; 
	long N = 0; // la valeur de stockage du resultat
	int nbDigits = 0; // nb de digits déja frappé
	int d = 0; // le digit courant
	do
	{ d = key.getDigitFmt(); // on va chercher la touche enfoncée 
		if ((d > toucherien) & (d <=touchezero)) // il sagit d'un symbole numerique
		{
			N = N * 10 + table[d - 1]; // on decale et ajoute la bonne valeur
			if (!((table[d - 1] == 0) && (nbDigits == 0) && prisezero) ) // on ne prends pas les zero en debut de nombre sauf si demande
			{
				nbDigits++; // nb de digits pris en compte
				digitalWrite(ledcontrol,true);
				Serial.print(table[d - 1]); // visualisation
				digitalWrite(ledcontrol,false);
			}
		}
	}
	while ((nbDigits < nbdigitsmax) & (d <=touchezero)); 
	if ((N == 0) && (nbDigits == 0)) // cas extreme on a tappé sur '.' ou enter
	{
		Serial.print('0');
	}
	return N;
}

// *********************************************************
// prend la valeur entière et la valeur décimale d'un nombre
// gère les cas <enter>, .xy<enter> , 0.<enter>, .<enter> ...

void prendNombre(long &ent, long &dec, byte nent, byte ndec)
{
	Serial.print('[');
	ent = prendEntier(nent, true); // prise de la valeur entiere
	int der = key.getDerniereTouche(); // pour savoir commenton continu
	switch (der)
	{
	case valide : // enter enfoncé
		{
			dec = 0;
			break;
		}
	default: // derniere touche 1..10 et nombre nent atteint ou point decimal enfoncé
		{
			Serial.print('.');
			dec = prendEntier(ndec, false);
			break;
		}
	}

	Serial.println(']');
}
// **************************************************


// initialisation 

void setup()
{
	Serial.begin(vitessevisu); // pour l'affichage sur la console
	key.begin(); // prise en compte tension alim (si aucunes touches n'est enfoncée)
	pinMode(ledcontrol,OUTPUT); // led de validation de la touche

}

// Procedure principale appelée de manière cyclique
void loop()
{

	long e=0; // la valeur entiere
	long d=0; // la valeur decimale
	prendNombre(e, d, nbavantvirgule, nbdecimale);
	// on reforme le réel à partir de la valeur entière et de la partie decimale
	float r=d;
	// prise en compte du decalage de la partie decimale
	for (byte i=0; i<nbdecimale;i++) {r=r/10;}
	// on ajoute la partie entiere
	r=r+e;
	// on affiche
	Serial.print("valeur reelle : "); Serial.println(r,nbdecimale); 
};
