
// Bibliotheque vkey pour gérer un clavier Sparkfunk
// Vkey Voltage Keyboard
// qui permet sur une seule entrée annalogique d'avoir 12 touches 'tout ou rien'
// Bibliotheque ecrite par Cordier Yves le 18/03/2017
// Libre de droit
// V1.0 le 18/03/2017
// V1.1 le 06/12/2017 ajout de getDigit


// entete fichier Vkey.cpp
#include "vkey.h"

// *************************************************************
// Definition des procedures
// *************************************************************
// Le constructeur

Vkey::Vkey(int pin)
{
	pinkey = pin; // broche de raccordement
	derniereTouche = 0; // par defaut
	derniereToucheValidee = 0; // par defaut;
	tempsDernierAppuis = millis();
	setRepetition(); // 500 ms par defaut
	setMaxRepAvantAcc(); // 5 par defaut
	setPasValeur(); // valeur par defaut (correspond à 5v);
	dernierAppel = tempsDernierAppuis; // pour gerer les repetitions
	nbRepetition = 0; // valeur initiale
}

// *************************************
// prend en compte la touche appuyée sur front

int Vkey::getDigitFmt()
{ long t = millis();
	if ( (t - dernierAppel) < 20) {
		delay(20); // pb de temps de reponse du clavier
	}
	dernierAppel = millis();
	int num = round(float( analogRead(pinkey)) / pasValeur);
	if (num > 0)
	{
		num = 13 - num; // la touche 12 est la premiere ;
		derniereToucheValidee = num;
	}

	if (num != derniereTouche)
	{
		derniereTouche = num;
		if (num != 0)
		{
			tempsDernierAppuis = millis();
		}
		nbRepetition = 0;
		return num; // sortie avec la valeur num
	}
	else
	{
		long tempsActuel = millis();
		long deltat = tempsActuel - tempsDernierAppuis;
		if (((nbRepetition < maxRepetitionAvantAcceleration) && (deltat >= tempsRepetition))
				||
				(nbRepetition >= maxRepetitionAvantAcceleration) && (deltat >= tempsRepetition / 10))
		{
			tempsDernierAppuis = tempsActuel;
			nbRepetition++;
			return num; // sortie avec la valeur num
		}
	}
	return 0; // sortie avec rien
}

// *************************************
// prend en compte la touche appuyée 

int Vkey::getDigit()
{ long t = millis();
	if ( (t - dernierAppel) < 20) {
		delay(20); // pb de temps de reponse du clavier
	}
	dernierAppel = millis();
	int num = round(float( analogRead(pinkey)) / pasValeur);
	if (num > 0)
	{
		num = 13 - num; // la touche 12 est la premiere ;
		derniereToucheValidee = num;
	}

	derniereTouche = num;  
	nbRepetition = 0;
	return num; // sortie avec la valeur num
}
// *************************************
// Change la valeur du temps de répétition
void Vkey::setRepetition(long r)
{
	tempsRepetition = r;
}
// *************************************
// Change la valeur du nombre de repetitions avant acceleration
void Vkey::setMaxRepAvantAcc(int n)
{
	maxRepetitionAvantAcceleration = n;
}

// *************************************
// Change la valeur du pas numerique entre touches
void Vkey::setPasValeur(int p)
{
	pasValeur = p;
}
// *************************************
// Change la valeur du pas numerique entre touches
int Vkey::getDerniereTouche()
{
	return derniereToucheValidee;
}
// *************************************
// initialisation de parametres (necessaire pour cartes alimentées autre qu'en 5v)

void Vkey::begin() // au demarrage 
{
	// a l'initialisation, on suppose qu'aucune touche est enfoncée
	// on lit la valeur et on en déduit le pas entre les touches
	// tension  valeurinit  pas
	// 5.0v       9         42
	// 3.3v      14         59
	// en supposant le phénomene linéaire,  pasValeur =3.4*valeurinit+11.4

	// pasValeur = round(3.4 * analogRead(pinkey) + 11.4);
	// Methode pas assez fiable 

	// Methode proposée : on se limite à deux tensions 5v et 3.3 
	// un seuil de 11 au depart semble cohérent et fiable.
	if (analogRead(pinkey) > 11) 
	{
		pasValeur = 59; // alimenté en 3.3v
	} 
	else
	{
		pasValeur = 42; // alimenté en 5v
	}
}
// fin du fichier vkey.cpp
